#ifndef _3DTYPES_H
#define _3DTYPES_H

#include	"3dmath.h"				//	For 3d math data structures
#include	"application.h"			//	need some info for the viewport class


/********************************************************************
*	3dtypes.h														*
*	this file contains classes for implementing all kinds of 3d		*
*	types like lights, materials, textures, fog, etc..				*
*	using the direct3d pipeline										*
********************************************************************/

// this useful macro.. for making the materials colors
#define RGBAWORD(r,g,b,a)  (b+((g)<<8)+((r)<<16)+(a<<24))
#define RGBCOLOR(r,g,b,a)  color4(RGBAWORD(r,g,b,a))

//	includes for OpenGL
#include "opengl.h"
#include "framecontroller.h"


/********************************************************************
*	Lights Structs hierarchy										*
*	the base struct is sLIght and from it the 3 light types			*
*	are inherited													*
*	By	Shady El Mously												*
********************************************************************/

#define SLIGHT_RANGE_MAX      ((float)sqrt(3.402823466e+38F))



/********************************************************************
*	struct sLight declaration										*
*	this is an abstract class, no object from its type can ever		*
*	exist															*
********************************************************************/

struct sLight 
{
protected:

	GLenum		m_GLid;					//	the number of the color
	static int	m_Base;					//	base starts from zero


	//	constructor made protcted to be used only by derived classes
	sLight	(
				const color3&	diff	=	color3::White,
				const color3&	spec	=	color3::Black,
				const color3&	amb		=	color3::Black
			);

public:

	color4		dcvDiffuse;				//	diffuse color of the light
	color4		dcvSpecular;			//	specular color of the light
	color4		dcvAmbient;				//	ambient color of the light
	point3d		dvDirection;			//	direction of the  light
	point4d		dvPosition;				//	position of the  light

	//	destructor decrements number of lights
	virtual ~sLight()	
	{
		Deactivate();		m_Base--;
	}

	// turn on the light, virtual, varies from light to the other
	virtual void Activate();

	//	deactivation of lights is same for all
	void Deactivate()
	{
		glDisable(GL_LIGHT0+m_GLid);	
	}
};
/*------------------end of struct sLight declaration---------------*/


/********************************************************************
*	struct sDirectionalLight declaration							*
*	represents a directional light									*
********************************************************************/
struct sDirectionalLight : public sLight
{
	//	constructor of the light, sets all the components
	sDirectionalLight(
						const point3d& dir,
						const color3&	diff	=	color3::White,
						const color3&	spec	=	color3::Black,
						const color3&	amb		=	color3::Black
						);

	//	activates the directional light
	void Activate();
};
/*-------------end of struct sDirectionalLight declaration----------*/

/********************************************************************
*	struct sPointLight	declaration									*
*	represents a point light in the scene							*
********************************************************************/
struct sPointLight : public sLight
{
	float		dvRange;				//	the range of the light, will be always maximal
	float		dvAttenuation0;			//	constant attenuation factor
	float		dvAttenuation1;			//	Linear attenuation factor
	float		dvAttenuation2;			//	quadratic attenuation factor

	//	construct with all required settings
	sPointLight		(		
					const point3d& loc,
					const color3&	diff	=	color3::White,
					const color3&	spec	=	color3::Black,
					const color3&	amb		=	color3::Black
					);

	//	activates the point source light
	void Activate();
};

/*---------------end of struct sPointLight declaration--------------*/


/********************************************************************
*	struct sSpotLight declaration									*
*	represents a spot light											*
*	this is the most complex light structure						*
********************************************************************/

struct sSpotLight : public sLight
{
	float		dvRange;				//	range of the spotlight, will be maximal
	float		dvAttenuation0;			//	constant attenuation factor
	float		dvAttenuation1;			//	Linear attenuation factor
	float		dvAttenuation2;			//	quadratic attenuation factor
	float		dvTheta;				//	angle of the spot light


	//	construct with all settings
	sSpotLight	(
				const point3d& loc,
				const point3d& dir,
				float theta,
				const color3&	diff	=	color3::White,
				const color3&	spec	=	color3::Black,
				const color3&	amb		=	color3::Black
				);

	//	activate the spot light
	void Activate();
};
/*----------------end of struct sSpotLight declaration-------------*/



/********************************************************************
*	class material													*
*	By	Shady El Mously												*
*	this class encapsulates the D3DMATERIAL struct					*
*	it is used to represent a material of an object					*
*	when activated all rendering will use it						*
********************************************************************/
struct sMaterial
{
	color4	diffuse;
	color4	ambient;
	color4	specular;
	color4	emissive;
	float	power;
	bool	m_bIsActive;		//	true when the material is active


public:
	//	default constructor
	//	a white material
	sMaterial(
		color4& amb		=	color4(1.0f,1.0f,1.0f,1.0f),
		color4& diff	=	color4(1.0f,1.0f,1.0f,1.0f),
		color4& spec	=	color4(1.0f,1.0f,1.0f,1.0f),		
		color4& emit	=	color4(0.0f,0.0f,0.0f,0.0f),
		float pow=1)
	{
		Set(amb,diff,spec,emit,pow);
		m_bIsActive	=	false;
	}

	//	destructor	deactivated the material
	virtual ~sMaterial()
	{
		Deactivate();
	}

	// changing the material
	void Set(
		color4& amb	=	color4(1.0f,1.0f,1.0f,1.0f),
		color4& diff	=	color4(1.0f,1.0f,1.0f,1.0f),
		color4& spec	=	color4(1.0f,1.0f,1.0f,1.0f),
		color4& emit	=	color4(0.0f,0.0f,0.0f,0.0f),
		float pow=1);

	// make the material the active one
	void Activate();


	//	disable the material
	void Deactivate()
	{
		if(m_bIsActive)
		glPopAttrib();	//	restore material state
	}


	// predefined standard materials to be used
	static const sMaterial AQUA_GLAZE;
	static const sMaterial AQUA_GLAZE_METAL;
	static const sMaterial BARREL_BLUE;
	static const sMaterial BEIGE_MATTE;
	static const sMaterial BEIGE_PLASTIC;
	static const sMaterial BLACK_MATTE;
	static const sMaterial BLACK_PLASTIC;
	static const sMaterial BLUE_BUMPY_CLOTH;
	static const sMaterial BLUE_METAL_SKY;
	static const sMaterial BLUE_PLASTIC;
	static const sMaterial BRASS;
	static const sMaterial BRUSHED_METAL;
	static const sMaterial CEILING_TILE;
	static const sMaterial CHERRY_RED_METAL;
	static const sMaterial CHROME_BLUE_SKY;
	static const sMaterial CHROME_MOROCO;
	static const sMaterial CHROME_PEARL;
	static const sMaterial SKY_BLUE_METAL;
	static const sMaterial OPENGL_DEFAULT;
};

/*-----------------------------------------------------------------*/
//		end of class material declaration



/********************************************************************
*	class	camera													*
*	By	Shady El Mously												*
*	This class represents a view point from which the user			*
*	Looks into the world											*
*	it is an encapsulation of the view matrix stuff					*
********************************************************************/
class cCamera
{
public:


	//	the camera vectors
	point3d m_Loc;				//	location of the viewer
	point3d m_At;				//	what the viewer looks at..
	point3d m_Up;				//	world up vector

	point3d m_Fwd;				// Forward (Vector) 
	point3d m_Side;				// Side (right)


	// Orientation data
	// Yaw, Pitch, and Roll
	float	m_Yaw		;		//	orientation in the direction of nodding your head (no)
	float	m_Pitch		;		//	orientation in the direction of nodding your head (yes)
	float	m_Roll		;		//	the last orientation ..


	// Forward Speed
	float	m_Speed;

	//	the time delta
	float&	TimeDelta;

public:

	//	make a camera with a location and euler angles
	cCamera(point3d& loc,float yaw,float pitch,float roll);

	//	create an abstarct camera
	cCamera():TimeDelta(FrameController()->m_DeltaTime)
	{	Init(point3d(0,0,0),0,0,0);	}

	//	initialize the data of the camera..the location and the angles
	void Init(point3d Loc,float Yaw,float Pitch,float Roll);

	//	compute the camera info accoring to the angles and the location
	void ComputeInfo();

	//	calling this will make the activate the camera
	void Activate()	;
	
	//	camera control functions..

	//	control the camera with keyboard
	void KeyBoard();
	
	//	move the camera..relative to the world coordinates
	void Translate(float x,float y,float z);

	//	move relative to viewer point...
	void TranslateLocal(float x,float y,float z);

	//	this will rotate locally..
	void RotateLocal(float yaw,float pitch,float roll);
};

/*-----------------------------------------------------------------*/
//		end of class camera declaration

/********************************************************************
*	class viewport													*
*	By Shady el Mously												*
*	this class enpasulates the D3DVIEWPORT7 functionality			*
********************************************************************/
struct sViewPort
{
	DWORD	dwX;
	DWORD	dwY;            /* Viewport Top left */
	DWORD	dwWidth;
	DWORD	dwHeight;       /* Viewport Dimensions */
	float	dvMinZ;         /* Min/max of clip Volume */
	float	dvMaxZ;

public:
	//constructor, use set parameters or use full screen by default
	sViewPort(int startx=0,int starty=0,int width=Application()->m_Width,int height=Application()->m_Height)
	{Set(startx,starty,width,height);}

	//	destructor resets the viewport to full screen
	virtual ~sViewPort()
	{Deactivate();}

	//	This will set the viewport parameters
	void Set(int startx,int starty,int width,int height);

	//	activate the viewport 
	void Activate(){glViewport(dwX,Application()->m_Height-dwY-dwHeight,dwWidth,dwHeight);}

	//	activate the viewport 
	void Deactivate(){glViewport(0,0,Application()->m_Width,Application()->m_Height);}
};


/*-----------------------------------------------------------------*/
//		end of class viewport declaration



enum texTypes_t
{
	PCX,
	BMP,
	TGA
};


// only partial pcx file header
typedef struct
{
	unsigned char manufacturer;
	unsigned char version;
	unsigned char encoding;
	unsigned char bits;
	unsigned char xMin;
	unsigned char yMin;
	unsigned char xMax;
	unsigned char yMax;
	unsigned char *palette;
} PCXHEADER;

/********************************************************************
*	class cBitmap													*
*	By Shady EL Mously												*
*	represents a 2d image											*
*	which could be loaded from a 24 bit rgb windows bmp file		*
*	or a TGA (targa file)											*
*	using those objects must be while in 2d mode					*
*	(after calling Graphics()->InitDrawing2D()						*
********************************************************************/
class cBitmap
{
protected:


	unsigned char	m_BitCount;			//	bit count per pixel


	texTypes_t textureType;

	int				m_RGBTYPE;		//	GL_RGB for bitmaps and GL_RGBA for pcx


	//	used for pcx files
	int				m_ScaleWidth;
	int				m_ScaleHeight;
	unsigned char	*m_Palette;



public:

	unsigned char*	m_pImageData;		//	pointer to image data
	int				m_Width;			//	Image	Width
	int				m_Height;			//	Image Height


	//	initialize local data items
	void Init(void);

	//	constructor set everything to zero
	cBitmap()
	{
		Init();
	}
	//	construct from a bitmap file
	cBitmap(char* filename)
	{
		Init();
		Load(filename)		;
	}

	//	Draw the bitmap where its down left corner in the sent x,y coordinates of the view port
	void Draw(int x,int y)
	{
		glRasterPos2i(x,y);
		glDrawPixels(m_Width,m_Height,m_RGBTYPE,GL_UNSIGNED_BYTE,m_pImageData);
	}

	

	//	destructor should free the image data
	virtual ~cBitmap()
	{
		SAFE_FREE(m_pImageData);
	}

	//	load the bitmap from a windows bmp file
	void LoadBMPFile(char* filename);

	//	Load Image from a targa file
	int	LoadTGAFile(char* filename);

	//	the terain class needs to acces private data here
	friend struct sTerrainMesh;

	//	swap Red and Blue
	void SwapColors(void* buffer,int size);


	//	load any file type
	void Load(char* fileName);


	//	load pcx file
	void LoadPCXFile(char *filename);


	//	get pcx data
	unsigned char *LoadPCXData(char *filename, PCXHEADER *pcxHeader);

};
/*----------------end of class cBitMap declaration-----------------*/

/****************************************************************
*	class cTexture	declaration									*
*	By Shady El Mously											*
*	Part of the flulib class library							*
*	this class is another texture class							*
*	but this one uses bmp files									*
*	it should be more standrd than the texture one				*
****************************************************************/
class cTexture : public cBitmap
{
private:


protected:

	cTexture(){}

public:

	//	the texture object name
	unsigned int	m_TextureName;

	//	claywa construct wioth a 24 bit bitmap
	cTexture(char* filename);

	//	call to actiavte the texture
	void Activate()
	{	
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D,m_TextureName);		//	set a texture
	}

	//	call when done using the texture
	void Deactivate()
	{
		glDisable(GL_TEXTURE_2D);
	}


	//	for special effects, we may render to a texture

	//	start rendering to the texture
	void StartDrawing();

	//	finish redering on the texture
	void EndDrawing();




	friend class cMultiTexture;
};
/*-----------end of class cTexture declaration---------------------*/


/********************************************************************
*	class cMultitexture												*
*	creates a texture												*
*	from 3 bitmap files combined, can be used to create				*
*	lots of great effects											*
*	when using multitexting, lighting calculations are cancelled	*
*	WARNING: The Display driver must support multitextring in order	*
*	to use this class, otherwise... the program will crash			*
********************************************************************/
class cMultiTexture : public cTexture
{
private:

	cTexture	m_Texture1;		//	the first texture
	cTexture	m_Texture2;		//	the second texture
	int			m_GenMode;		//	generation mode for the 2nd texture, may be
								//	GL_OBJECT_LINEAR or GL_SPHERE_MAP or GL_EYE_LINEAR




public:

	//	constructor will create 2 textures
	//	and creates the multitextring stuff if required
	cMultiTexture(char* Tex1name,char* Tex2name,int GenMode=GL_OBJECT_LINEAR);


	//	must be de activated on destruction
	virtual ~cMultiTexture()
	{	Deactivate();	}


	//	activate the multitextiring
	void	Activate();

	//	deactive after drawing with it
	void	Deactivate();
};

/*------------end of class cMultitexture declaration---------------*/
/********************************************************************
*	class cEnvtexture												*
*	represnts and enviromental texture map							*
*	used to simulated shiny objects that reflects everything from	*
*	around them														*
*	the only difference is in the activate/deactivate functions		*
*	which will make the texture coordinated automaticly calculated	*
********************************************************************/
class cEnvTexture : public cTexture
{
private:

public:

	cEnvTexture(char* Tex1name) : cTexture(Tex1name)
	{/*-------all done---------*/}

	//	activate the multitextiring
	void	Activate();

	//	deactivate
	void Deactivate();
};


/*------------end of class cEnvtexture declaration---------------*/

/********************************************************************
*	class cEnvPasstexture											*
*	the same but using blending										*
********************************************************************/
class cEnvPassTexture : public cTexture
{
private:

public:

	cEnvPassTexture(char* Tex1name) : cTexture(Tex1name)
	{/*-------all done---------*/}

	//	activate the multitextiring
	void	Activate();

	//	deactivate
	void Deactivate();
};


/*------------end of class cEnvtexture declaration---------------*/

/********************************************************************
*	class cLineartexture											*
*	represnts and linear texture map								*
*	no texture coordinates are needed with this texture				*
********************************************************************/
class cLinearTexture : public cTexture
{
private:

public:

	cLinearTexture(char* Tex1name) : cTexture(Tex1name)
	{/*-------all done---------*/}

	//	activate 
	void	Activate();

	//	deactivate
	void Deactivate();
};


/*------------end of class cLineartexture declaration---------------*/

/****************************************************************
*	class cMirrorFloor											*
*	By Shady El Mously											*
*	used to simulate a flat rectangualr mirror					*
*	using both of the stencil buffer and alpha blending			*
*	techniques													*
*	this class works better in 32 bit colors					*
*	the mirror is assumed to be located at the origin,			*
*	flatened on the x-z plane									*
*	to make sure the mirros works correctly i should repeat		*
*	the same sequence when drawing the mirror image				*
****************************************************************/
class cMirrorFloor
{
private:

	


public:

	cTexture	m_MirrorTexture	;		//	mirror texture

	float		m_Width			;		//	mirror width
	float		m_Height		;		//	mirror height
	float		m_Transparency	;		//	transparecny value of the mirror


	cMirrorFloor(char * filename): m_MirrorTexture(filename)
	{
		m_Transparency	=	0.4f;
		m_Width			=	4.0f;
		m_Height		=	4.0f;
	}
	//	call this and before drawing the reflected scene
	void StartDrawingReflecion(void);

	//	call this when done drawing the reflected scene
	void EndDrawingReflection(void);

	//	this will draw the quad mirror itself
	void Draw();
};
/*------------end of class cFlatMirror declaration-------------*/


/********************************************************************
*	class fog														*
*	By	Shady El Mously												*
*	this class implements the fog duntionality of Direct3d			*
********************************************************************/


class cFog
{
private:
	float	m_Start;	//	where the fog starts from the viewpoint
	float	m_End;		//	where the fog ends from the viewpoint
	float	m_Density;	//	density value used in the 2 exponential fog modes
	color3	m_Color;	//	the fog color
	GLint	m_Type;		//	the fog type


public:
	// construct the fog using its colors, range start and end and type
	cFog(const color3& Color,float Start=0,float End=5,int Type=GL_LINEAR)
	{	Set(Color,Start,End,Type);	}

	// set the fog parameters and type..
	void Set(const color3& Color,float Start,float End,int Type=GL_LINEAR,float Density=1.0f);

	// this will activate the fog
	void Activate()
	{	glEnable(GL_FOG);	}

	// will deactiavte the fog
	void Deactivate()
	{	glDisable(GL_FOG);}
};

/*-----------------------------------------------------------------*/
//		end of class fog declaration




#endif // _3DTYPES_H
