#ifndef _3DUI_H
#define _3DUI_H

/****************************************************************
*	flulibui.h													*
*	by Shady El Mosly											*
*	part of the flulib class library							*
*	this is the definition file for some user interface objects	*
*	buttons,menus,slides.. in 2d and in 3d						*
****************************************************************/
#include	"physics.h"
#include	"simulators.h"
#include	"audio.h"


/****************************************************************
*	class GLFont												*
*	encapsulates opengl text functionality						*
****************************************************************/
struct GLFont
{

	GLuint	m_ListBase;
	int		m_Widths[256];
	int		m_Height;

	//	constructor.. does nothing
	GLFont()
	{}

	//	destroy the font
	virtual ~GLFont()
	{	Destroy();	}

	void Create(const char*,int,int,DWORD);

	//	destroy the font
	void Destroy();

	//	put a normal string
	void Puts(const char* s);

	//	put a formatted string
	void Printf(int,const char*,...);
};
/*-----------------end of GLFont declaration-------------------*/


/****************************************************************
*	class cCursor												*
*	the cursor is controlled using direct input..				*
*	input should be catched before processing the cursor		*
*	with any button												*
****************************************************************/

class cCursor : public cBitmap
{
public:

	point2d	m_Loc;

	//	the controllers are public to be able to reset them and change the parameters
	cSecondOrder<int>		m_Xcontrol;		//	X controller of the cursor
	cSecondOrder<int>		m_Ycontrol;		//	Y controller of the cursor


	//	construct with a bitmap as the cursor icon..
	cCursor(char * IconFileName);

	//	Set The Cursor Position...
	void SetPos(int X,int Y)	{	m_Loc.x	=	X;		m_Loc.y	=	Y;	}

	//	processes the cursor movement from the direct input mouse object
	//	i should ctach the couse state before calling this function
	void Draw();
};


/*-------------------end of class cursor-----------------------*/


/****************************************************************
*	class cButton												*
*	by shady el mously											*
*	this class implements buttons, can be used for menus etc..	*
****************************************************************/
class cButton
{
protected:
	
	point2d			m_Position,m_Dimensions	;	// position and dimensions
	cBitmap	*		m_Image					;	//	if it uses a bitmap, a pointer to it
	bool			m_bWasUnderMouse		;
	

public:

	bool		m_bIsUnderMouse;				// is the mouse over it ?
	bool		m_bIsHolded;					// is the mouse holded on it ?


	VoidFunction	m_Function				;	//	void function to be called when the button is clicked

	// public constructor
	cButton()	{Init();}					// default constructor..does nothing
	cButton(char * label,char * bmpfilename=NULL);


	void Init();							//	initialization function
	void SetImage(char * filename);			//	sets a bitmap file to be used..
	
	//	destructor.. virtual for future inheritance
	virtual ~cButton()	
	{	SAFE_DELETE(m_Image);	}

	//	this will center the button on the width of the screen
	void CenterVertically(void)
	{	m_Position.x=Application()->m_Width/2-m_Dimensions.width/2;	}
	// is it clicked?? TRUE OR FALSE
	bool IsClicked(void);
	//	small inline methods..
	//	this will set the button position
	void SetPos	(int x,int y){	m_Position.x=x;			m_Position.y=y;		}
	//	will process and draw the button
	void Process();


};
//	end of class button definition
/*-------------------------------------------------------------*/

/****************************************************************
*	class c3DCursor												*
*	represents a 3d mouse cursor								*
*	completly controlled using directinput						*
*	and may be used with the 3d buttons and menus				*
*	the response is second order								*
****************************************************************/
class c3DCursor : public cRigidBody
{
private:

	cSecondOrder <point3d>	controller;

public:

	//	always construct 3d cursors with response parameters
	c3DCursor(float wn,float zeta)
	{
		controller.SetParams(wn,zeta);	
	}

	//	draw the button according to its position and update it too
	void Draw();
};
/*---------end of class cursor3d specification-----------------*/

/****************************************************************
*	class button3d												*
*	By Shady El Mously											*
*	combining the button with the rigidbody using multiple		*
*	inheritance gives a 3d button								*
****************************************************************/
class c3DButton : public cButton,public cRigidBody
{
private:

	point3d		m_RestLoc			;		//	rest location of the button
	float		m_Angle				;		//	angle of the button around its Y axis(used in processing it)

public:

	string		m_Help				;		//	a string associated with the button


	//	default constructor
	c3DButton()
	{}

	//	this will process and draw the 3d button
	//	the button is processed with the x,y compoinet of a 3d mouse cursor..
	void Draw();

	//	the 3d menu should be able to access everything here
	friend class c3DMenu;
};

/*---------end of class c3DButton declaration------------------*/

/****************************************************************
* class c3DMenu													*
*	this is an array of buttons..								*
*	to simplify using more than one button						*
****************************************************************/
class c3DMenu : public vector<c3DButton *>
{
private :

	FILE*	m_Fptr;	//	file pointer fo reading the 3d menu description file


	//	functions required while reading the menu description file
	bool IsComment();
	void PrepareLine();
	bool NewSection();
	void ReadMenu();
	void ReadButton();


	vector<cString>&	m_LineWords;		//	alias for the global array of string
	string				m_MenuName;			//	the name of the menu

public:


	vector<string>		m_SceneNames;	//	scene names from this menu
	vector<c3DButton*>	m_SceneButtons;	//	a vector containing pointers to the buttons
										//	that do actiavte scenes, pointers here should not be deallocated
	

	c3DButton *	m_pExitButton	;	//	pointer to an exit button ( or return or cancel )

	//	always construct with a text file describing the menu
	c3DMenu(char* FileName);

	//	destructor will destroy all memory used by buttons
	virtual ~c3DMenu()
	{
		for (int i=0;i<size();i++)
			delete (*this)[i];
	}

	//	draw the whole menu
	void Draw();
};
/*---------end of class button 3d specification----------------*/

/****************************************************************
*	class c3DSlide												*
*	By Shady El Mously											*
*	this class represents a bitmap..on a slide					*
*	the slide has a water mesh and is correctly textured		*
*	and scaled													*
****************************************************************/
class c3DSlide : public cRigidBody
{
public:


	//	construct with a bitmap, this time the texture will be owned by the slide.. it will destruct it
	c3DSlide(char* filename,int NumVerts=30);


	//	Splash The water
	void Splash(point3d& loc, float pwr)
	{
		if(m_Mesh)((sWaterMesh*)m_Mesh)->Splash(loc,pwr);
	}

	//	simulates the water effects
	void SimulateWater(float Speed)
	{
		if(m_Mesh)((sWaterMesh*)m_Mesh)->Process(Speed);
	}

	//	sets the dmaping of water
	void SetWaterDamping(float d)
	{
		if(m_Mesh)((sWaterMesh*)m_Mesh)->SetDamping(d);
	}
};
/*-----------end of class c3dslide declaration---------------------*/


/********************************************************************
*	class cMarquee													*
*	used to represent text scenes									*
*	sentence which will be on screen written from a text file		*
********************************************************************/
class cMarquee : public cThread
{
protected:

	vector<string>		m_SentencesList		;		//	all the sentences in the file...
	int					m_CurSentenceNum	;		//	indexing for the currsnt sentence
	bool				m_Finished			;

	char*				m_CurString			;		//	pointer to the current string
	int					m_Yposition			;		//	the vertical position of the text displayed
	int					m_Xposition			;		//	horizontal position of the string on the string

	DWORD				m_Color				;		//	color of the text
	
public:


	//	construct with a file containing all the text..
	//	each line will be considered a nsetence that will
	//	be displayed inidividually
	cMarquee(char* FileName);

	//	destructor
	~cMarquee()
	{	Kill();	}

	//	the run function, which will change the current sentence based on some sort of timing
	void Run(void);

	//	Draw the marquee text..this should be called from the outside thread
	void Draw();

	//	return true when the text is all done
	bool Done()
	{	return m_Finished;	}

	//	terminate the marquee
	void Kill()
	{	m_Finished	=	true;	}
};
/*---------------end of class cMarquee declaration-----------------*/


/********************************************************************
*	class cScrollMarquee											*
*	inherited from cMarquee, makes the marquee text scroll			*
*	from left to right on the screen								*
********************************************************************/
class cScrollMarquee : public cMarquee
{
private:
	deque<char>			m_VisibleText		;	//	a double ended queue will contain all the visible text

public:

	//	always construct marquues with text files
	cScrollMarquee(char* FileName);

	//	the run for scrolling marquee
	void Run();
};
/*--------------end of class cScrollMarquee declaration------------*/


/********************************************************************
*	class cMatrixMarquee											*
*	this makes the text of marquee be typed character by character	*
*	on screen														*
********************************************************************/
class cMatrixMarquee: public cMarquee
{
private:

	cSound *	m_pTypeSound;	//	sound played when each charcater is typed
public:

	//	matrix marquee constructor
	cMatrixMarquee(char* FileName,char* TypeSoundFileName=NULL): cMarquee (FileName),m_pTypeSound(NULL)
	{
		//	All Done !
		if(TypeSoundFileName)
			m_pTypeSound	=	new cSound(TypeSoundFileName);
	}

	//	destructor safely get rid of the sound
	~cMatrixMarquee()
	{	SAFE_DELETE(m_pTypeSound);	}

	//	the thread for the matrix marquee
	void Run();
};
/*---------------end of class cMatrixMarquee declaration-----------*/

#endif //_3DUI_H