#ifndef _AI_H
#define _AI_H
/****************************************************************
*	ai.h														*
*	implementation of artificial intelligence behaviour			*
*	motivation, steering and decision making					*
****************************************************************/

#include "physics.h"		//	here we take physical rigid  bodies and give them some mind
#include "text.h"

#define REALLY_BIG	999999.9f
#include <stack>		//	for the Djikstra's algorithm
#include <map>			//	for the neural network class
using std::stack;
using std::map;

/****************************************************************
*	class cSmartObject											*
*	this class simulates some sort of intelligent behavior		*
*	inherited from the body3d or rigis..						*
*	it can make object react with each other.. chase..			*
*	evade.. etc	i cane give this capabilty to rigid				*
*	this will give some smart behviour to them					*
*****************************************************************/
class cSmartObject : public cRigidBody
{
private:

	//	Potential function constant
	const static float m_PotFuncConstant;

public:

	float		m_Speed;	//	this is the speed of resopnse
	
	
	//	this contrcutor is for the body3d and bsphere3d
	cSmartObject()	{Init();}


	//	initialization function set a speed of 0.005, no pattern..
	void Init();
	//	destructor frees memory..
	~cSmartObject(){}

	//	terminator chasing , very simple..
	void Chase(cRigidBody&	other);

	//	this will evade another smartobject
	void Evade(cRigidBody&	other);

	
	//	chase one and evade the others as they are obstacles (potential functions)
	template <class T>
	void Chase(cRigidBody& other,T* Obstacles,int NumObstacles)
	{

		if(bsphere3d::intersect(*this,other))		return;
		//	first get vector from object to target
		point3d goal=other.m_loc-m_loc;
		point3d dirvec=goal/goal.mag();
		//	for each obsatcle
		for(int i=0;i<NumObstacles;i++)
		{
			//	if it isnt itself
			if((this)!=&Obstacles[i])
			{
				//find the vector between the creature and the object
				point3d ObstacleVec=m_loc-Obstacles[i].m_loc;
				//	find the value to subtract for each obstacle
				float dist=ObstacleVec.mag()-Obstacles[i].m_radius-m_radius;
				//	this is the vector pointing away from the obstacle
				ObstacleVec.normalize();
				dirvec+=ObstacleVec*(m_PotFuncConstant/(dist*dist));
			}
		}
		dirvec.normalize();
		Translate(dirvec*m_Speed);
	}
};
/*------------------end of class cSmartObject------------------*/

/****************************************************************
*	class cCharacter											*
*	used to represent charcaters								*
*	loaded from md2 files										*
*	the characters can be in many states						*
*	and the mesh is always from the md2 type					*
****************************************************************/


// character states
enum CharacterState
{
	MODEL_IDLE,		// idle animation
	MODEL_CROUCH,		// crouch animation
	MODEL_RUN,		// running animation
	MODEL_JUMP		// jumping animation
};


class cCharacter : public cSmartObject
{

private:

	CharacterState	m_State;

public:

	//	always construct with an md2 file and its texture
	cCharacter(char* modelFileName,char* TextureFileName=NULL);

	// set animation state of model
	int SetState(CharacterState state);	

	//	this will draw the character in its state
	void Draw();

	// retrieve animation state of model
	CharacterState GetState();
};



/*------------end of class cCharacter declaration--------------*/


/********************************************************************
*	class file														*
*	encapsulating all the functions that could be made				*
*	on a file for reading/writing..									*
********************************************************************/

class cFile  
{
protected:

	FILE* m_fp;
	std::string m_filename;

public:

	cFile();
	~cFile();

	// Discard if exists
	void Create( const char* filename );

	// Append if exists, fail if not
	void Append( const char* filename );

	// Read if exists, fail if not
	void Open( const char* filename );

	// Close the file
	void Close();

	// Check to see if a file exists
	static bool Exists( const char* filename );

	// Read directly out of the file 
	bool ReadBuff( void* pBuffer, int buffSize );

	// Write directly out to the file 
	void WriteBuff( void* pBuffer, int buffSize );

	// Read in a line of text from the file, assuming we're passed a big enough buffer
	// this can be dangerous.  buffers passed in should be >= 255 in length, to be safe
	void ReadLine( char* pBuffer );
	std::string ReadLine();

	// Convenience function.  filles 'pBuffer' with the next line that doesn't start 
	// with the provided comment character
	void ReadNonCommentedLine( char* pBuffer, char commentChar );

	// Takes the next line of non-commented text and parses it using " \t\n" 
	// as delimiters
	void TokenizeNextNCLine( std::queue< std::string >* pList, char commentChar = '#' );

	// so basic structures can be automatically read and written
	template <class type>
	void Read( type* data )
	{
		ReadBuff( data, sizeof( type ) );
	}

	template <class type>
	void Write( type& data )
	{
		WriteBuff( &data, sizeof( type ) );
	}
};
/*-------------------end of class file declaration-----------------*/



/****************************************************************
*	class neural network										*
*	a generic neural network used to simulate intelligence		*
*	the network can be constructed from a file describing the	*
*	nodes and the edges											*
****************************************************************/

class cNeuralNet
{
protected:
	class cNode;
	class cEdge;

public:
	string GetOutput();

	void SendInput( const char* inputName, float amt );
	
	void Load( cFile& file );
	void Run();
	void Clear();

	cNeuralNet()
	{
		// no work needs to be done.
	}

	virtual ~cNeuralNet();

	void ListInputs();
	void GetInputs();

protected:

	cNode* FindNode( const char* name );

	class cNode
	{
	public:
		void Init( const char* name, float weight );

		void Clear();
		virtual void Run();

		void AddOutEdge( cNode* target, float edgeWeight );
		void SendInput( float in );

		const char* GetName() const;
		float GetTotal() const;
	protected:

		// Computes the output function given the total.
		virtual float CalcOutput();
		
		string	m_name;
		float	m_weight; // initial bias in either direction
		float	m_total;  // total of the summed inputs up to this point

		vector< cEdge >	m_outEdges;

	};

	class cEdge
	{
		cNode*	m_pSrc;
		cNode*	m_pDest;
		float	m_weight;
	public:

		cEdge( cNode* pSrc, cNode* pDst, float weight );

		void Fire( float amount );
	};
	friend class cNode;

	vector< vector< cNode* > > m_nodes;

	// maps the names of output nodes to output strings.
	map< string, string > m_inputs;
	map< string, string > m_outputs;

	string m_defaultOutput;
};


inline const char* cNeuralNet::cNode::GetName() const
{
	return m_name.c_str();
}

inline float cNeuralNet::cNode::GetTotal() const
{
	return m_total;
}

/*----------end of class neural network declaration------------*/


/****************************************************************
*	class cNeuralThred											*
****************************************************************/
class cNeuralThread : public cThread
{
private:

	char m_NNFileName[200];


public :

	cNeuralThread(const char *FileName)
	{
		strcpy(m_NNFileName,FileName);
	}

	void Run(void)
	{
		cNeuralNet nn;
		cFile file;
		file.Open(m_NNFileName);
		nn.Load(file);
		file.Close();
		char str[200];

		int done = 0;

		Text()->PutString( "Neural Net Simulator\n");
	
		while( !done )
		{
			// Clear the totals
			nn.Clear();

			// List the inputs for the net from the user
			nn.ListInputs();

			// Get the inputs for the net from the user
			nn.GetInputs();

			// Run the net
			nn.Run();

			// Get the net's output.
			string output = nn.GetOutput();

			sprintf(str,"\nNeural Net output was [%s]\n", output.c_str() );
			Text()->PutString(str);
			sprintf(str,"\nRun Again? (y/n)\n");
			Text()->PutString(str);
			char buff[80];
			Text()->GetString(buff);
			Text()->Clear();

			if( !(buff[0] == 'y' || buff[0] == 'Y') )
			{
				done = 1;
			}
		}
	}
};



#endif