/************************************************************
*	flulibapp.cpp											*
*	By Shady El Mously										*
*	3D Application class implementation						*
*	here lies the application constructor and destructor	*
************************************************************/


#include "application.h"			//	Header implemented here
#include "text.h"					//	Text I/O			SubSystem 
#include "opengl.h"					//	OpenGL				SubSystem
#include "framecontroller.h"		//	Synchornization		Subsystem
#include "input.h"					//	DirectInput			SubSystem
#include "audio.h"					//	DirectSound			SubSystem
#include <time.h>					//	time info needed to make the CurTime Function
#include <ddraw.h>					//	for detecting display modes

/************************************************************
*	Implementation of the cApplication class				*
************************************************************/

//	a vector holding all the possible display modes supported..
vector <sScreenMode> cApplication::m_AvailableModes;

//	application constructor..
//	some data should be read from a settings file first
//	after that all application SubSystems are created
cApplication::cApplication(HINSTANCE instance)
{
	m_Language		=	ENGLISH;
	m_IsDead		=	false	;
	m_KeyPressed	=	false	;
	m_InThread		=	false	;
	m_FatalError	=	false	;
	m_FrameRate		=	0		;
	m_Instance		=	instance;

	//	try to load previously saved settings
	if(!LoadSettings())		SetDefaultSettings();

	//	now i create the main application window where
	//	this will simply register the class with a message handler..
	m_MainWindow=new cWindow("Graphics Window",MessageHandler);

	//	this will actually create the window..
	//	if full screen it will be a popup window..
	if(m_IsWindowed)
		m_MainWindow->Create(m_Width,m_Height);
	else
		m_MainWindow->Create(m_Width,m_Height,WS_POPUP|WS_VISIBLE);

	//	store the window handle to be global
	m_IsActive			=	true;

	//	full screen apps doesnt need hte mouse cursor to be visible anymore..
	//if(!m_IsWindowed)ShowCursor(FALSE);

	//m_Menu=LoadMenu(instance,MAKEINTRESOURCE(IDR_MENU1));
	//SetMenu(GetHwnd(),m_Menu);
	


	m_ReportFile.open("success.txt");
	//	now start all subsystems..
	Mouse()				=	new		sMouseInfo;
	Graphics()			=	new		cOpenGLSubSystem;
	Text()				=	new		cTextSubSystem;
	FrameController()	=	new		cFrameController;
	Input()				=	new		cInputSubSystem;
	Audio()				=	new		cAudioSubSystem;
	//	in some apps.. create a menu from resources
}


//	application destructor.. shuts down all subsystems 
//	and savesettings
cApplication::~cApplication()
{
	DestroyMenu(m_Menu);
	//	shut down all opened subsystems..
	SAFE_DELETE	(      Audio()		);
	SAFE_DELETE	(      Input()		);
	SAFE_DELETE	( FrameController() );
	SAFE_DELETE	(      Text()		);
	SAFE_DELETE	(    Graphics()		);
	SAFE_DELETE	(     Mouse()		);

	//	destroy the main window..application finished
	SAFE_DELETE(m_MainWindow);

	//	and save the settings into an ini file..in case of no error
//	if(!m_FatalError)	SaveSettings();
}



//	this will load the setting from an external file 
//	if the file is not found it returns false
bool cApplication::LoadSettings(void)
{

	char dummy[20];
	//	try to open a seetings file..
	ifstream settingsfile;

	settingsfile.open("settings.ini");

	//	if no such file exists.. return false
	if(!settingsfile)
		return false;

	settingsfile>>dummy>>dummy>>m_Width;
	settingsfile>>dummy>>dummy>>m_Height;
	settingsfile>>dummy>>dummy>>m_BitDepth;
	settingsfile>>dummy>>dummy>>m_TripleBuffer;
	settingsfile>>dummy>>dummy>>m_IsWindowed;

	return true;
}

//	this function is called when setting file is not found
//	it sets some default value for application
//	parameters and asks the user for windowed/fullscreen choice..
void cApplication::SetDefaultSettings(void)
{
	//	Default Safe Values are 640*480*16
	m_Width				=	640;
	m_Height			=	480;
	m_BitDepth			=	16;
	m_IsWindowed		=	true;
	m_TripleBuffer		=	false;

	//	popup the configure dialog
	DialogBox(m_Instance,MAKEINTRESOURCE(CFG_DIALOG),NULL,ConfigDialogProc);
}


//	this function will save the settings into a file (settings.ini)
//	it will be called from the destructor usually...
void cApplication::SaveSettings(void)
{
	ofstream OutFile;
	OutFile.open("settings.ini");
	OutFile<<"WIDTH    = "<<m_Width<<endl;
	OutFile<<"HEIGHT   = "<<m_Height<<endl;
	OutFile<<"BITDEPTH = "<<m_BitDepth<<endl;
	OutFile<<"TBUFFER  = "<<m_TripleBuffer<<endl;
	OutFile<<"WINDOWED = "<<m_IsWindowed<<endl;
}

//	this DDCallBack function will be called for every possible screen mode
//	it will save the data in the static vector of the application class
HRESULT WINAPI EnumDisplayModesCallBack(LPDDSURFACEDESC ddsd,LPVOID ScreenModesVector)
{
	vector<sScreenMode>* Modes=(vector<sScreenMode>*)ScreenModesVector;

	sScreenMode NewMode;
	NewMode.BitDepth	=ddsd->ddpfPixelFormat.dwRGBBitCount;
	NewMode.Height		=ddsd->dwHeight;
	NewMode.Width		=ddsd->dwWidth;

	//	only add non 8 bit modes...
	if(NewMode.BitDepth!=8)
	Modes->push_back(NewMode);

	return DDENUMRET_OK;
}


//	the dialog handler function
//	the dialog pops up when is there is no previously saved settings
//	and it is used to set the application setting for display modes..(until now)
BOOL CALLBACK cApplication::ConfigDialogProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	//	a temporary directdraw object to check modes with...
	LPDIRECTDRAW tempDD;
	int i;
	switch (msg)
	{
	case WM_INITDIALOG:
		
		// if you can't create the main dd object.. write error and exit..
		// do not try if it already exists
		if(DirectDrawCreate(NULL,&tempDD,NULL)!=DD_OK)
			Application()->ReportError("error creating main direct draw interface");

		//	i can enumerate the display modes here..
		tempDD->EnumDisplayModes(NULL,NULL,&m_AvailableModes,EnumDisplayModesCallBack);

		//	done.. now release the temp dd object
		SAFE_RELEASE(tempDD);

		//	add the windowed string to the list box
		SendDlgItemMessage(hwnd,IDC_LIST2,LB_ADDSTRING,0,(DWORD)(LPSTR)"Windowed");
		
		//	add all the other display modes to the list box...
		for(i=0;i<m_AvailableModes.size();i++)
		{
			char str[100];
			sprintf(str,"%dx%dx%d",m_AvailableModes[i].Width,m_AvailableModes[i].Height,m_AvailableModes[i].BitDepth);
			SendDlgItemMessage(hwnd,IDC_LIST2,LB_ADDSTRING,0,(DWORD)(LPSTR)str);
			//	if it is 640*480*16..select it...
			if((m_AvailableModes[i].BitDepth==32||m_AvailableModes[i].BitDepth==24)&&m_AvailableModes[i].Height==480&&m_AvailableModes[i].Width==640)
			SendDlgItemMessage(hwnd,IDC_LIST2,LB_SELECTSTRING,0,(DWORD)(LPSTR)str);
		}
		return TRUE;
		break;

	case WM_COMMAND:
		int id=LOWORD(wParam);
		if(id==IDOK)
		{
			//	now check the select item in the list box..
			int SelMode;
			SelMode=SendDlgItemMessage(hwnd,IDC_LIST2,LB_GETCURSEL,0,0L);
			//	if zero is selected... this is the windowed app...640*480..
			if(SelMode==0)
			{
				Application()->m_IsWindowed	=	true;
				Application()->m_Width		=	640;
				Application()->m_Height		=	480;
			}
			//	else set the display mode from the available modes..
			//	subtract one from the SelMode..
			else
			{
				Application()->m_IsWindowed	=	false;
				Application()->m_Width		=	m_AvailableModes[SelMode-1].Width;
				Application()->m_Height		=	m_AvailableModes[SelMode-1].Height;
				Application()->m_BitDepth	=	m_AvailableModes[SelMode-1].BitDepth;
			}
		}
		if(id==IDEXIT||id==IDCANCEL)			Application()->ReportError("Bye Bye");
		//	end dialog on ok or cancel...
		if (id==IDOK || id==IDCANCEL||id==IDEXIT)			EndDialog(hwnd,id);
		break;
	}
	return FALSE;
}

//	this will return true when its time to terminate everything
bool cApplication::ProcessMessages()
{
	if(PeekMessage(&Application()->m_Msg,NULL,0,0,PM_REMOVE))
	{
		if(m_Msg.message==WM_QUIT)			return true;
		TranslateMessage	(&m_Msg);
		DispatchMessage		(&m_Msg);
	}
	else
	if(m_IsDead)
		return true;
	
	return false;
}


//	function report error									
//	when this is called, the application will shut down		
//	safely and the error will be reported in a text file	
//	before leaving											
void cApplication::ReportError(const char * str,...)
{

	m_FatalError	=	true;
	ofstream ErrorFile;

	va_list	ap;					//	argument pointer
	unsigned char s[1024];		//	output string

	//	make the formatted string
	va_start(ap,str);
	vsprintf((char*)s,str,ap);
	va_end(ap);


	// write the default error file for appending
	if(str)
	{
		ErrorFile.open("error.txt");
		ErrorFile<<s<<" at "<<CurTime()<<endl;
	}
	//	now shut down everything and leave with a message box
	//	throw the error jump into the catch block
	throw cError((char*)s);
}


//	gets the time and date when called as a c string
char * cApplication::CurTime()
{
	struct tm * ptr;
	time_t lt;
	lt=time(NULL);
	ptr=localtime(&lt);
	return asctime(ptr);
}

//	in multilanguage apps.. this function will return the correct filename if required
void cApplication::GetFullFileName(char * FileName,char* FullFileName)
{
	//	get the full file name..
	switch(Application()->m_Language)
	{
	case ENGLISH:
		sprintf(FullFileName,"./artwork/english/%s",FileName);
		break;
	case ARABIC:
		sprintf(FullFileName,"./artwork/arabic/%s",FileName);
		break;
	case FRENCH:
		sprintf(FullFileName,"./artwork/french/%s",FileName);
		break;
	}
}

//	this a message handler to be used with the window
//	in which the graphics layer will operate
HRESULT CALLBACK cApplication::MessageHandler(HWND hwnd,UINT message,WPARAM wparam,LPARAM lparam)
{
	switch(message)
	{
	//	this message will ensure the main message loop will break;
	case WM_DESTROY:
		Application()->m_IsDead=true;
		break;
	case WM_ACTIVATE:								// Watch For Window Activate Message
		if (wparam==0)						// Check Minimization State
			Application()->m_IsActive=false;					// Program Is Active
		else
			Application()->m_IsActive=true;					// Program Is No Longer Active
		return 0;
		break;
	case WM_MOVE:
		//	that was the only solution  to the problem which was here..
		if(Graphics())			Graphics()->UpdateRects();
		return 0;
			break;
	case WM_KEYDOWN:
		switch(wparam)
		{
		case VK_ESCAPE:
			//SendMessage(hwnd,WM_DESTROY,0,0);
			if(Application()->m_IsActive)
				Application()->m_IsDead=true;
			return 0;
			break;
		default:
			break;
		}
	default:
		break;

	}
return DefWindowProc(hwnd,message,wparam,lparam);
}

//	pops up a dialog for browsing..it will return the filename of the selected file
char* cApplication::OpenNewFile(const char *wndTitle )
{
    static char file[256];
    static char fileTitle[256];
    static char filter[] = "h3d file (*.h3d)\0*.h3d\0""All Files (*.*)\0*.*\0";

    OPENFILENAME ofn;

    lstrcpy( file, "");
    lstrcpy( fileTitle, "");

    ofn.lStructSize       = sizeof(OPENFILENAME);
    ofn.hwndOwner         = GetHwnd();
    ofn.hInstance         = (HINSTANCE) GetWindowLong(GetHwnd(),GWL_HINSTANCE);
    ofn.lpstrFilter       = filter;
    ofn.lpstrCustomFilter = (LPSTR) NULL;
    ofn.nMaxCustFilter    = 0L;
    ofn.nFilterIndex      = 1L;
    ofn.lpstrFile         = file;
    ofn.nMaxFile          = sizeof(file);
    ofn.lpstrFileTitle    = fileTitle;
    ofn.nMaxFileTitle     = sizeof(fileTitle);
    ofn.lpstrInitialDir   = NULL;
    ofn.lpstrTitle        = wndTitle;
    ofn.nFileOffset       = 0;
    ofn.nFileExtension    = 0;
    ofn.lpstrDefExt       = "*.*";
    ofn.lCustData         = 0;

    ofn.Flags = OFN_SHOWHELP | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

    if (GetOpenFileName(&ofn))
        return (char*)ofn.lpstrFile;

	
    else
        return NULL;
}
/*-----------------end of class appinfo implementation------------*/


