#ifndef _GRAPH_H
#define _GRAPH_H
#include "base.h"
#include "3dmath.h"

/********************************************************************
*	Graph.h															*
*	By Shady El Mously												*
*	This file is the header implementation for graph solving		*
*	algorithms,														*
*	classes here are used to make a 2d road map between nodes		*
*	linking some nodes .. makin a graph.. and finding paths from	*
*	one node to another inside the graph							*
********************************************************************/


//	make a vector of bool a new typedef... used in making 2d arrays
typedef vector<bool> BoolVect;


/********************************************************************
*	class cSearchState												*
*	represnts a state while searching inside the map				*
*	or in other words, represents a node in the search tree			*
*	which will be constructed while searching						*
*																	*
*	some data items here are dependant on the problem we are		*
*	trying to solve	which are the current node number and the		*
*	costtogoal and the current cost									*
*																	*
*	the rest of data items are things which will exist in any node	*
*	in any tree														*
*																	*
*	1.Pointer to parent node										*
*	or in other words a pointer to the state which led				*
*	to this state which will be null for the root node				*
*																	*
*	2.vector(array with data items from zero to infinity) pointers	*
*	to child nodes...												*
*	or in other words pointers to all the states which can be		*
*	reached from this state, also.. for leaves, this array			*
*	will be empty													*
********************************************************************/
struct cSearchState
{
	int		m_CurNode;		//	index that says which node in the graph this state represents
	float	m_CostToGoal;	//	this is the heuristic to goal
	float	m_CurCost;		//	cost accumulated so far


	//	now for the stuff which must be in any node in the world
	cSearchState*			m_ParentState;		//	a pointer to the parent state (Search state)
	vector<cSearchState*>	m_ChildStates;		//	a zero or more pointers to child states...(nodes)


	//	this static function is used to determine which state is better then the other
	//	it will be used to sort the nodes while searching
	static bool SortRule(cSearchState* a,cSearchState* b)
	{
		return (a->m_CurCost+a->m_CostToGoal)<(b->m_CurCost+b->m_CostToGoal);			
	}

	//	this fuinction is also used to determine which states are in the node inside the graph
	static EqualityRule(cSearchState* a,cSearchState* b)
	{	return a->m_CurNode==b->m_CurNode;	}


	//	constructor, just make the parent node pointer null..
	//	this wil be modified later whn the tree is made
	//	the root node is the only node which will have this pointer still NULL
	cSearchState()
	{		m_ParentState	=	NULL;	}


	//	the magic destructor..
	//	now each node when destructed
	//	must destroy all its children
	//	this means that destroying the root node mean the whole tree is destroyed
	virtual ~cSearchState()
	{
		for(int i=0;i<m_ChildStates.size();i++)
			delete m_ChildStates[i];
	}

	//	this function will be used to check if the indx sent came in any other parent node of
	//	this node..
	//	in other words, it will check if the state was previously rached up in the tree
	bool IsChildOf(int i);
};
/*--------------------end of class cSearchState--------------------*/

/********************************************************************
*	class cGraph													*
*	now the real confusion will start								*
*	this class will represent a whole graph ...						*
*	or, a road map.. a set of nodes .. or cities, with links		*
*	connecting some nodes with others								*
*	this class will have functions to add nodes to a graph			*
*	and linking the nodes togther									*
*	a map contain some nodes and some links							*
*	the nodes are a simple array of the nodes..						*
*	and the links are described using a 2D array of values which	*
*	will be ones in links.. ex: if m_Links[2][1] is true			*
*	this means the nodes 2 and 1 are linked togther					*
*																	*
*	and the graph itself has a pointer to a Searchstae				*
*	which will be used to construct the search tree while applying	*
*	the different search algorithms									*
********************************************************************/
class cGraph
{
protected:

	vector<point3d>		m_Nodes;		//	the nodes in the graph
	vector<BoolVect>	m_Links;		//	a matrix describing links between nodes

public:


	//	User Acces functions..
	void AddNode(point3d);			//	Add a new to the graph at position X,Y
	bool LinkNodes(int src,int dst);	//	Link nodes src and st in the map, they start from 0
	bool DropNode(int index);			//	Drop a node in the graph
	bool Exist(int i)					//	this function checks if a node index in valid
	{	
		if(i<0||i>m_Nodes.size()-1)
			return false;
		return true;
	}
	//	this function returns the floating point distance between 2 nodes..
	float DistanceBetween(int src,int dst);

	//	the tree must be able to have full access to things inhere
	friend class cSearchTree;

	cGraph()
	{}

	virtual ~cGraph()
	{}

};
/*------------------------end of struct sGraph---------------------*/

/********************************************************************
*	class cSearchTree												*
*	represents a tree of states										*
*	must be used with an existing graph to search in it				*
*	the tree has a root node, which is initially NULL				*
*	which can e used to construct trees								*
********************************************************************/
class cSearchTree
{
private:

	cSearchState	*	m_pRootNode;	//	pointer to root node of search tree
	cGraph			&	m_RoadMap;		//	the road map, or graph, must be sent when creating


	//	the internal insane algorithms..
	cSearchState*	BreadthFirst(int src,int dst);
	cSearchState*	DepthFirst(int src,int dst);
	cSearchState*	AStar(int src,int dst);

public:

	//	initially.,., there is no tree, i must use an existing graph to search in it
	cSearchTree(cGraph& map): m_RoadMap(map)
	{	m_pRootNode	=	NULL;	}
	
	//	destrcutor destroy the root node (if it was used) which will result in the whole tree destrcution
	virtual ~cSearchTree()
	{	SAFE_DELETE(m_pRootNode);	}


	//	function used externally to get the solutions
	vector<int>		BreadthFirstPath(int src,int dst);
	vector<int>		DepthFirstPath	(int src,int dst);
	vector<int>		AStarPath		(int src,int dst);
};
/*--------------------end of class cSearchTree---------------------*/
#endif