#ifndef _MESHES_H
#define _MESHES_H

/********************************************************************
*	meshes.h														*
*	By Shady El Mously												*
*	part of the flulib engin										*
*	this file is the header for the meshes used to create 3d object	*
*	a huge inheritance and lots of complex relations exists between	*
*	classes written here											*
********************************************************************/

#include "3dtypes.h"



/********************************************************************
*	3D objects classes definition									*
*	By Shady El Mously												*
********************************************************************/


/********************************************************************
*	i start by a mesh												*
*	and abstarct mesh with a number of vertices..					*
*	this is the base class for strided and indexed meshes..			*
*	the mesh has a name, and it has an array of vertices			*
*	which include normal and 2 texture coordiantes..				*
********************************************************************/
struct sMesh
{
	//	mesh data, vertices and triangles indeces
	vector<point3d>		m_Verts		;	//	those are the vertices forming the object
	vector<point3d>		m_Norms		;	//	those are the Normals on the object
	vector<texcoord>	m_TexCoords	;	//	first set of texture coordinates
	string				m_Name		;	//	name of the mesh
	cTexture*			m_Texture	;	//	texture associated here


	//	constructor does nothing
	sMesh();


	virtual ~sMesh()
	{	SAFE_DELETE(m_Texture);	}		//	destructor made virtual for child classes


	// this will get the bounding sphere radius
	float GenRadius();

	//	this will draw the mesh.. Transforms should be set before calling
	virtual void Draw(){}

	//	this will shift the mesh in the local object space
	virtual void Translate(float x,float y,float z);

	//	this will twist the mesh around a certain vector..
	virtual void Twist(float angle,float axeX,float axeY,float axeZ);

	//	scaling the normals is essential when scaling object matrices..
	virtual void ScaleNormals(float Ratio);

	//	scaling the normals is essential when scaling object matrices..
	virtual void ScaleNormals(float sX,float sY,float sZ);

};
/*---------------------------end of struct mesh------------------------*/
/************************************************************************
*	strided mesh														*
*	this time there...strided triangles, used to create spheres and		*
*	walls and any other strided meshes									*
************************************************************************/
struct sStridedMesh :public sMesh
{
	//	constructor..check if we support locking arrays and sets the essentail rendering function
	sStridedMesh();

	//	drawing a strided mesh using vertex arrays
	void Draw();

	//	this will create a sphere
	void CreateSphere(float radius,int numSphereRings,int numSphereSegments);
};

/*---------------------end of struct stridedmesh-----------------------*/
/************************************************************************
*	indexed mesh														*
*	a mesh with tringle indexes.										*
*	meshes of this type can be constructoed from o3d files				*
*	and also can be created as boxes and as liquid surfaces				*
*	it includes function to calculate the normals						*
*	and generating meshes on demand..									*
************************************************************************/
struct sIndexedMesh : public sMesh
{
	vector<stri>	m_Tris;			//	those are theindex to points making the triangles

	//	default constructor 
	sIndexedMesh();

	//	this will construct amesh from an o3d file..text file must termainte with t
	sIndexedMesh(const char * filename);

	//	calculate vector normals for indexed meshes
	void ComputeNormals(vector<point3d>& points,vector<point3d>& norms);

	//	draw the mesh as indexed..using the m_Tris array
	void Draw();

	//	this will create a box...
	void CreateBox(float Width,float Height,float Depth,int numsegx,int numsegy,int numsegz,int numfaces);

	//	this will twist the mesh around a certain vector..
	void Twist(float angle,float axeX,float axeY,float axeZ);

protected:

	//	read data from file using file pointer, file should be already opened..
	void	ReadMesh(FILE * file,bool binary);
};
/*----------------------end of struct indexedmesh----------------------*/

/************************************************************************
*	struct HierMesh														*
*	represents a hierarchical mesh										*
*	the structure is a tree structure									*
************************************************************************/
struct sHierMesh : public sIndexedMesh
{
	sHierMesh	*m_pNextMesh;	//	next mesh starts from in identity transform
	sHierMesh	*m_pChildMesh;	//	child meshes tranforsm are multiplied by their parent one
	sHierMesh	*m_pParentMesh;	//	pointer to mesh higher in hierarchy


	matrix4		m_CurMatrix;	//	current matrix transforming this mesh
	sMaterial	m_Material;		//	material in this node
	bool		m_bHasAlpha;	//	true is alpha blending is used here

	//	always construct with an h3d file
	sHierMesh(char* filename=NULL);

	//	destructor recursively destroys the tree
	~sHierMesh()
	{	SAFE_DELETE(m_pNextMesh	);	SAFE_DELETE(m_pChildMesh);	}

	//	loads the data from a file...binary or text...
	void LoadSingleMeshData(FILE * fptr,bool binary=false);

	//	recursive function that will read the hierarchy..
	void LoadHierarchy(FILE* fptr,bool binary);

	//	this will scale all the object normals by recusring inside the tree..
	void ScaleTree(float sX,float sY,float sZ);

	//	loads a hierarchical file in the tree mesh
	void Load(char* filename);

	//	the virtual drawing function..
	void Draw();

	//	get address of a mesh with	a certain name
	sHierMesh	*	GetMeshNamed(char* MeshName);

	//	scaling the normals is essential when scaling object matrices..
	void ScaleNormals(float Ratio)
	{	ScaleTree(Ratio,Ratio,Ratio);	}

	//	scaling the normals is essential when scaling object matrices..
	void ScaleNormals(float sX,float sY,float sZ)
	{	ScaleTree(sX,sY,sZ);	}
};
/*---------------end of hierarchical mesh declaration-------------------*/


/************************************************************************
*	watermesh															*
*	inherited from indexed mesh											*
*	this is a very special class										*
*	to create meshes that simulates dynamic flat surfaces				*
*	like water... flags...etc.											*
*	the mesh must be indexed and must contain more info					*
*	because it needs to be accessed as a 2d array of vertices..			*
************************************************************************/
struct sWaterMesh: public sIndexedMesh
{
	point3d	m_Dims;				//	dimensions of the mesh
	int m_Width,m_Height;		//	width and height of the net
	point3d	m_Step;			//	increment (sqaure area of each quad
	point3d	m_Start;		//	point to start from


	float	**			m_Heights1		;
	float	**			m_Heights2		;
	float	**			m_DampRatios	;



	//	constructore creates the 2d mesh...
	sWaterMesh(float Width,float Height,int numColumns,int numRows);


	//	default constructor (makes the pointers safe)
	sWaterMesh();

	//	destroy the vectors...
	~sWaterMesh();

	//	draw the water mesh as quads
	void	Draw();

	//	used to access the damping ratios
	float	DampRatio(int x,int y)
	{	return m_DampRatios[x][y];	}

	//	this will set the damping ratio of the water..
	void SetDamping(float value);

	//	setup some random values for ther surface items..
	//	negative or psitive.. send the maximim value of Z
	void Randomize(float range);

	//	this will make a splash 
	void Splash(point3d& loc,float power);

	//	this function is called per frame to ensure the mesh will act as water exactly !
	//	the speed of the effect and the damping ratio should be sent both..
	void Process(float speed);

	//	return the point at the specified position
	float& HeightAt(point3d& loc);

	// use this to scale the model..
	void Scale(float amt);
};
/*----------------------end of struct sWaterMesh-----------------------*/


/************************************************************************
*	struct sTerrainMesh													*
*	By Shady El Mously													*
*	represents a landscape terrain										*
*	with varying heights												*
*	the terraing must be generated from a 24 bit bitmap					*
*	which will create the varying heights according to the gray scale	*
*	values of the points												*
*	drawing the Terrains with their same Texture without lighting		*
*	produces the most beautifull view of the 3d map						*
************************************************************************/
struct sTerrainMesh : public sWaterMesh
{
private:

	int			m_ListID;		//	display list ID

public:

	//	construct from a 24 bit grayscale bimtap file..
	//	this will create a 3d terrain using a gray scale bitmap
	//	send the requred dimensions.. and the bitmap filename
	//	and the ratio by which the height values will be scales
	sTerrainMesh(float Width,float Depth,char* GrayScaleMapFileName,float HeightScale);


	//	draw..
	void Draw();


	//	send an address of a water mesh to the terrain and it will adjust the dampings to zero over the hight
	void MakeSea(sWaterMesh* pWater,float SeaLevel);


	~sTerrainMesh();
};
/*----------------------end of struct sTerrainMesh----------------------*/

/************************************************************************
*	struct sQuadricMesh													*
*	encapsulates the OpenGL Quadric,									*
*	used as a base class for the disk,sphere and cylinder meshes		*
************************************************************************/

struct sQuadricMesh: public sMesh
{
protected:

	//	the quadric object
	GLUquadricObj	*	m_Quadric;


	

public:

	//	constrcutor always makes the quadric object
	sQuadricMesh();

	//	destructor deletes the object
	~sQuadricMesh()
	{
		gluDeleteQuadric(m_Quadric);
	}

	virtual void Draw()=0;	
	
};
/*----------------end of struct sQuadricmesh---------------------------*/



/************************************************************************
*	struct sDiskMesh													*
************************************************************************/

struct sDiskMesh	: public sQuadricMesh
{
private:

	float m_InnerRadius;
	float m_OuterRadius;

	//	this will draw the quadric as a disk
	void Draw();

public:

	sDiskMesh(float innerRadius,float outerRadius):m_InnerRadius(innerRadius),m_OuterRadius(outerRadius)
	{	}
};
/*---------------------end of sDiskMesh declaration--------------------*/

/************************************************************************
*	struct sCylinderMesh												*
*	can also be used to create cones									*
************************************************************************/

struct sCylinderMesh	: public sQuadricMesh
{
private:

	float	m_BaseRadius;
	float	m_TopRadius;
	float	m_Height;

	
	//	this will draw the quadric as a disk
	void Draw();


public:

	//	always construct with the cylinder parameters
	sCylinderMesh(float baseRadius,float topRadius,float height):m_BaseRadius(baseRadius),m_TopRadius(topRadius),m_Height(height)
	{}
};
/*-------------------end of sCylinderMesh declaration--------------------*/

/************************************************************************
*	struct sSphereMesh													*
************************************************************************/
struct sSphereMesh	: public sQuadricMesh
{
private:

	float	m_Radius;

	//	this will draw the quadric as a disk
	void Draw()
	{
		gluSphere(m_Quadric,m_Radius,20,20);
	}

public:

	//	always construct with the cylinder parameters
	sSphereMesh(float radius):m_Radius(radius)
	{}
};
/*-------------------end of sSphereMesh declaration--------------------*/
/************************************************************************
*	struct sTorusMesh													*
************************************************************************/
struct sTorusMesh	: public sQuadricMesh
{
private:

	float	m_Radius1;
	float	m_Radius2;

	//	this will draw the quadric as a disk
	void Draw();

public:

	//	always construct with the cylinder parameters
	sTorusMesh(float radius1,float radius2):m_Radius1(radius1),m_Radius2(radius2)
	{}
};
/*-------------------end of sSphereMesh declaration--------------------*/

/************************************************************************
*	struct sTextMesh													*
*	represent a 3d text object											*
*	always constructed from a string									*
************************************************************************/

struct sTextMesh	:	public sMesh
{
private:

	char m_String[256];
	float length;
	int idx;	


public:

	//	store the string internally
	sTextMesh(char* text)
	{
		strcpy(m_String,text);
	}


	//	Draw the 3d text..
	void Draw();

};
/*----------------------end of struct sTextMesh declaration------------*/


/************************************************************************
*	struct sStarsMesh													*
*	imeplements a 3d object composed of random points					*
*	this object should always be drawn with no lighting					*
*	and with the polygon mode set to point								*
************************************************************************/

struct sStarsMesh : public sStridedMesh
{
private:

	void Init();
	int cnt;
	double radius;
	int glListID;

public:

	sStarsMesh(int c, double r=1.0): cnt(c), radius(r), glListID(0) 
	{
		Init();
	}
	void Draw() 
	{

		glCallList(glListID);
	}

};


/*----------------end of struct sStarsMesh declaration------------------*/
#endif // _MESHES_H