#ifndef _SPECIALEFFECTS_H
#define _SPECIALEFFECTS_H

#include "3dtypes.h"
#include "opengl.h"
/********************************************************************
*	special.h														*
*	By Shady EL Mously												*
*	Adding Special 3d effects to the engine							*
*	like billboarding,particle systems, reflexions and shadows		*
********************************************************************/
#define FRAND ((rand()%2000)-1000)/1999.0f 


//	bill boarding is drawing a single quad that is always facing the viewer may be useful in some cases
void BillBoardedQuad(point3d& pos,float size);

/********************************************************************
*	struct sParticle												*
*	represents a single particle in the particle system				*
********************************************************************/
struct sParticle
{
	point3d	m_Loc;			//	location of the particle
	point3d	m_PrevLoc;		//	previous location
	point3d	m_Velocity;		//	direction and speed
	point3d	m_Acceleration;	//	acceleration

	float	m_Energy;		//	determines how long the particle is alive
	float	m_Size;			//	size of the particle
	float	m_SizeDelta;	//	change in size over time
	float	m_Weight;		//	determines how gravity affects the particle
	float	m_WeightDelta;	//	change of it in time

	color4	m_Color;		//	current color of the particle
	color4	m_ColorDelta;	//	rate of change of color
};
/*-------------------end of struct particle------------------------*/




/********************************************************************
*	class cParticleSystem											*
*	represents a particle system									*
*	this is a base class from which childs can be implemented for	*
*	particle system based effects such as snow, smoke, fire,		*
*	explosions, blood..												*
********************************************************************/
class cParticleSystem
{
public:

	//	constructor
	cParticleSystem(int maxParticles,point3d& origin);

	virtual ~cParticleSystem(){}

	//	abstract functions
	virtual void Update(float elapsedtime)	=	0;
	virtual void Render()					=	0;

	virtual	int	Emit(int numparticles);
	virtual	void	InitializeSystem();
	virtual void	KillSystem();
	
	point3d		m_Origin	;			//	source location
	float		m_TimeScale	;			//	time scale for the particle system



protected:

	virtual void InitializeParticle(int index)	=	0;
	sParticle*	m_ParticleList;		//	particles for this emitter
	int			m_MaxParticles;		//	maximum number of particles
	int			m_NumParticles;		//	number of particles
	float		m_AccumulatedTime;	//	used to track how long since the last particle was emitted
	float		m_Force;			//	force(gravity..wind) acting on the particle system
};



/*-------------end of class cParticleSystem declaration------------*/

/********************************************************************
*	class cSnowStorm												*
*	represents a particle system simulating snowflass				*
*	snow flakes should be made from a bitmap file with a black		*
*	background														*
********************************************************************/


class cSnowStorm : public cParticleSystem
{
public :

	//	construct the snow storm by sending its origin , dimentions and the snowflake bitmap file
	cSnowStorm(int maxparticles,point3d& origin,float height,float width,float depth,char* filename);

	//	destructor just releases the texture
	virtual ~cSnowStorm()
	{	SAFE_DELETE(m_pTexture);	}

	//	update using the time passed
	void Update(float elapsedtime);

	//	this will render the storm
	void Render();

	//	use this to set the snowstorm attributes, call initialize after calling it for effect
	void SetAttributes(point3d& FlakeVelocity,point3d& VelocityVariation,float FlakeSize,float Rate);


protected :

	//	this will initialize a single particle
	void InitializeParticle(int index);


	float		m_Height;					//	height of snow field
	float		m_Width;					//	width of snow field
	float		m_Depth;					//	depth of snow field

	point3d		m_FlakeVelocity;			//	initial velocity of all flakes
	point3d		m_FlakeVelocityVariation;	//	variation of flakes velocity
	float		m_SnowFlakeSize;			//	snowflake size
	float		m_SnowFlakesRate;			//	snow flakes rate
	cTexture*	m_pTexture;					//	texture object

};

/*-----------------end of class cSnowStorm declaration--------------*/

/********************************************************************
*	class cFlame													*
*	a particle system which represnts a flame of particles			*
*	fading form a color into black									*
*	can be used to represent fire effects, smoke, flames			*
*	still neds more tweacking										*
********************************************************************/
class cFlame : public cParticleSystem
{
public:

	point3d	m_ParticleVelocity;				//	initial particle velocity
	point3d	m_ParticleVelocityVariation;	//	variation in the velocity
	float	m_ParticlesRate;				//	rate of flow of smoke particles
	color4	m_InitialColor;					//	initial color of particles
	float	m_Size;
	float	m_DeltaSize;
	float	m_MinSize;						//	minimum size of particles
	float	m_MaxSize;						//	maximum size of particles
	
	//	make the smoke
	cFlame(int numparticles,point3d& origin,color4 InitialColor,char* BmpFileName) :cParticleSystem(numparticles,origin)
	{
		m_ParticlesRate				=	2000							;
		m_pTexture					=	new cTexture(BmpFileName)		;
		m_InitialColor				=	InitialColor					;
		m_MinSize					=	0;
		m_MaxSize					=	1;
	}

	//	use this to set the parameters
	void SetParams(color4 InitialColor,point3d& ParticleVelocity,point3d& ParticleVelocityVariation,float StartSize,float SizeDelta);
	

	//	destructor just frees the texture
	virtual ~cFlame()
	{
		SAFE_DELETE(m_pTexture);
	}

	//	update using the time passed
	void Update(float elapsedtime);

	//	this will render the smoke
	void Render();

protected :

	cTexture	*	m_pTexture;		//	texture for a single smoke particle

	//	this will initialize a single particle
	void InitializeParticle(int index);
};
/*--------------------end of class cFlame declaration------------------*/
/************************************************************************
*	class cItems														*
*	this new particle system will also give acceleration				*
*	to the particles													*
************************************************************************/
class cItems : public cFlame
{
public:

	point3d	m_ParticleAcceleration;		//	initial particle acceleration

	//	make the smoke
	cItems(int numparticles,point3d& origin,color4 InitialColor,char* BmpFileName) ;

	//	initialize a single particle
	void InitializeParticle(int);

	//	the update
	void Update(float elapsedtime);


};

/*---------------end of class cItems declaration-----------------------*/

#endif