/****************************************************************
*	water.cpp													*
*	by Shady el mously											*
*	implementation of the water contianer structure				*
*	lots of math and physics and very advanced OPENGL commands	*
****************************************************************/
#include "meshes.h"
#include "water.h"




/****************************************************************
*			class sWaterContainer implementation				*
****************************************************************/



#define WATER_ASPECT 1.0f
#define ATI_SQRTOFTWOINV 4.94974747f


//	helper functions...used for texture matrix usage with water
void InvertAffine(float mat[16])
{
	mat[3] = mat[7] = mat[11] = mat[12] = mat[13] = mat[14] = 0.0f;
	swap(mat[1], mat[4]);
	swap(mat[2], mat[8]);
	swap(mat[6], mat[9]);
}


void SetTextureMatrix(float paraboloidOrientation)
{
   float po = paraboloidOrientation;

   float M[16];
   float P[16]  = {   1, 0, 0, 0,   0,   1, 0, 0,   0, 0, 1, 1,     0,   0, 0, 0  };
   float S[16]  = {  -1, 0, 0, 0,   0,  -1, 0, 0,   0, 0, 1, 0,     0,   0,po, 1  };
   float T[16]  = { 0.5, 0, 0, 0,   0, 0.5, 0, 0,   0, 0, 1, 0,   0.5, 0.5, 0, 1  };
   float C[16]  = {  po, 0, 0, 0,   0,  po, 0, 0,   0, 0, 1, 0,     0,   0, 0, po };
   float C2[16] = {   1, 0, 0, 0,   0,  0, -1, 0,   0, 1, 0, 0,     0,   0, 0, 1  };

   //Grab the modelview matrix
   glGetFloatv(GL_MODELVIEW_MATRIX, M);
   InvertAffine(M);

   //Change to texture matrix
   glMatrixMode(GL_TEXTURE);

   glLoadIdentity();
   glMultMatrixf(C);
   glMultMatrixf(T);
   glMultMatrixf(P);
   glMultMatrixf(S);
   glMultMatrixf(M);

   glMatrixMode(GL_MODELVIEW);
}




//	construct with the number of vertices on width and height, deph of container and a damping factor
sRealWater::sRealWater(int Width,int Height,float Depth,float Damp)
{
	m_pRefractionTexture=NULL;
	m_pReflectionTexture=NULL;

	//	store the data
	m_Width			=	Width				;
	m_Height		=	Height				;
	m_DampFactor	=	1.0f	-	Damp	;
	m_Depth			=	Depth				;
	m_Fresnel		=	true				;

	m_NumVertices = Width * Height;
	m_NumIndices = (Width-1) * (Height-1) * 4;

	//	create the 2 textures from default filenames
	m_pReflectionTexture	=	new cTexture("env.bmp");
	m_pRefractionTexture	=	new cTexture("refract.bmp");

	//	reserve memory for the arrays according to the size sent
	m_Verts.reserve(m_Width*m_Height);
	m_Norms.reserve(m_Width*m_Height);
	m_TexCoords.reserve(m_Width*m_Height);
	m_Colors.reserve(m_Width*m_Height*4);
	m_ForceArray.reserve(m_Width*m_Height);
	m_VelocityArray.reserve(m_Width*m_Height);
	m_Indices.reserve((m_Width-1) * (m_Height-1) * 4);

	//	clear the forces array
	memset(m_VelocityArray.begin(), 0, sizeof(float)*m_NumVertices);

	//	and initialize the water surface
	InitSurface();
}

//	destructore, get rid of the 2 textures
sRealWater::~sRealWater()
{
	SAFE_DELETE(m_pRefractionTexture);
	SAFE_DELETE(m_pReflectionTexture);
}


//	this function initialize the water surface, as a 1x1 square at the origin
void sRealWater::InitSurface(void)
{
	int j,k;
	int x,y;
	int *indexPtr;
	point3d dx, dy;
	point3d *vertPtr;

	//	calculates vertices
	vertPtr = m_Verts.begin();
	dx.x = ( 1.0f - -1.0f) / (m_Width-1);
	dx.y = (-1.0f - -1.0f) / (m_Width-1);
	dx.z = 0.0f;
	dy.x = (-1.0f - -1.0f) / (m_Height-1);
	dy.y = ( 1.0f - -1.0f) / (m_Height-1);
	dy.z = 0.0f;
	for (j=0; j<m_Height; j++)
	{
		for (k=0; k<m_Width; k++)
		{
		vertPtr->x = -1.0f + dx.x*(float)k + dy.x*(float)j;
		vertPtr->y = -1.0f + dx.y*(float)k + dy.y*(float)j;
		vertPtr->z = 0.0f + dx.z*(float)k + dy.z*(float)j;
		vertPtr++;
		}
	}

	//	calculate indices..we index in quads
	x = 0;
	y = m_Width;
	indexPtr = m_Indices.begin();
	for (j=0; j<m_Height-1; j++)
	{
		for (k=0; k<m_Width-1; k++)
		{
		indexPtr[0] = x;
		indexPtr[1] = x+1;
		indexPtr[2] = y+1;
		indexPtr[3] = y;
		indexPtr += 4;
		x++;
		y++;
		}
		x++;
		y++;
	}
}








//	this function will update the surface of the water using the 
//	water equation grabbed from the book
//	this method is different from the method in sWaterMesh, it looks much more realistic..
//	the euation uses forces and veloicities..
void sRealWater::UpdateSurface(void)
{
	float d;
	int x,y;
	float deltaTime;

	//	set all forces to be zero
	memset(m_ForceArray.begin(), 0, sizeof(float)*m_NumVertices);


	//	compute the forces ....
	//	pass on all vertices in the array
	//	and compute 8 values for the eight neibouring vertices
	//	the first 4 ( up,down,left,right)
	//	are the values of the up,down,left and right z values
	//	and the other 4 are the corners z values, multiplied by one over square root of 2
	//	so.. the computaion looks like this...
	//	
	//		1/sqrt(2)	|	1	|	1/sqrt(2)
	//		------------+-------+-------------
	//			1		|	0	|		1
	//		------------+-------+-------------
	//		1/sqrt(2)	|	1	|	1/sqrt(2)
	for (x=2; x<m_Height-2; x++)
	{
		for (y=2; y<m_Width-2; y++) 
		{
			d = m_Verts[m_Width*x + y].z - m_Verts[m_Width*x + y-1].z;
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*x + y-1] += d;

			d = m_Verts[m_Width*x + y].z - m_Verts[m_Width*(x-1) + y].z;
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*(x-1) + y] += d;

			d = (m_Verts[m_Width*x + y].z - m_Verts[m_Width*x + y+1].z);
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*x + y+1] += d;

			d = (m_Verts[m_Width*x + y].z - m_Verts[m_Width*(x+1) + y].z);
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*(x+1) + y] += d;

			d = (m_Verts[m_Width*x + y].z - m_Verts[m_Width*(x+1) + y+1].z)*ATI_SQRTOFTWOINV;
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*(x+1) + y+1] += d;

			d = (m_Verts[m_Width*x + y].z - m_Verts[m_Width*(x-1) + y-1].z)*ATI_SQRTOFTWOINV;
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*(x-1) + y-1] += d;

			d = (m_Verts[m_Width*x + y].z - m_Verts[m_Width*(x+1) + y-1].z)*ATI_SQRTOFTWOINV;
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*(x+1) + y-1] += d;

			d = (m_Verts[m_Width*x + y].z - m_Verts[m_Width*(x-1) + y+1].z)*ATI_SQRTOFTWOINV;
			m_ForceArray[m_Width*x + y] -= d;
			m_ForceArray[m_Width*(x-1) + y+1] += d;
		}
	}
	//	all forces at all points calculated !

	//	now calculate velocities according to those forces...just multiply by a constant 
	//	and add to the velocity array( which was initialized to zero
	//	also damp the water with the damping factor
	deltaTime = 0.005f;
	for (x=0; x<m_NumVertices; x++)
	{
		m_VelocityArray[x] += (m_ForceArray[x] * deltaTime);
		m_VelocityArray[x] *= m_DampFactor;
	}

	//	and finally, caluclate the new positions
	for (x=0; x<m_NumVertices; x++)
		m_Verts[x].z += m_VelocityArray[x];
}


//	this calculates the water surface normals
//	essential for the lighting effect..
//	by averaging the z values of the neighbouring vertices
void sRealWater::ComputeNormals(void)
{
	int i;
	int j;
	float tmpf;

	int m = m_Width;
	int n = m_Height;
   
	point3d *vert;
	point3d *normal;

	vert = m_Verts.begin();
	normal = m_Norms.begin();


	for (i=0; i<n; i++)
	{
		for (j=0; j<m; j++)
		{
			normal->x = 0.0f;
			normal->y = 0.0f;
			normal->z = 1.0f;

			//Above
			if (i != 0)
			{
				if (j != 0)
				{
					normal->x += -vert[-m-1].z;
					normal->y += -vert[-m-1].z;
				}
				else
				{
					normal->x += -vert[-m].z;
					normal->y += -vert[-m].z;
				}
				normal->x += -vert[-m].z*2.0f;
				if (j != m-1)
				{
					normal->x += -vert[-m+1].z;
					normal->y += vert[-m+1].z;
				}
				else
				{
					normal->x += -vert[-m].z;
					normal->y += vert[-m].z;
				}
			}
			else
			{
				normal->x += -vert[0].z;
				normal->x += -vert[0].z*2.0f;
				normal->x += -vert[0].z;

				normal->y += -vert[0].z;
				normal->y += vert[0].z;
			}

			//Current line
			if (j != 0)
				normal->y += -vert[-1].z*2.0f;
			else
				normal->y += -vert[0].z*2.0f;

			if (j != m-1)
				normal->y += vert[1].z*2.0f;
			else
				normal->y += vert[0].z*2.0f;

			//Below
			if (i != n-1)
			{
				if (j != 0)
				{
					normal->x += vert[m-1].z;
					normal->y += -vert[m-1].z;
				}
				else
				{
					normal->x += vert[m].z;
					normal->y += -vert[m].z;
				}
				normal->x += vert[m].z*2.0f;
				if (j != m-1)
				{
					normal->x += vert[m+1].z;
					normal->y += vert[m+1].z;
				}
				else
				{
					normal->x += vert[m].z;
					normal->y += vert[m].z;
				}
			}
			else
			{
				normal->x += vert[0].z;
				normal->x += vert[0].z*2.0f;
				normal->x += vert[0].z;

				normal->y += -vert[0].z;
				normal->y += vert[0].z;
			}
			//Normalize
			tmpf = 1.0f / (float)sqrt(normal->x*normal->x + normal->y*normal->y + 1.0f);
			normal->x *= tmpf;
			normal->y *= tmpf;
			normal->z *= tmpf;

			vert++;
			normal++;
		}
	}
}


//	this is the most complex function, it generates texture coordinates for the refacted interior
//	of the water container, is used the location of the camera
//	and the height of each vertex, to calculate the texture coordiantes
//	it also calculates fresnel terms which changes the color according to the viewer location 
void sRealWater::CalculateRefractionCoordinates(void)
{
	int i;
	int j;
	float mInc;
	float nInc;

	int m = m_Width;
	int n = m_Height;
   
	point3d *vert;
	point3d *normal;
	point3d camera;
	point3d pos;
	unsigned char *color;
	float *texCoord;


	float matrix[16];
	
	//Get camera position and orientation
	glGetFloatv(GL_MODELVIEW_MATRIX, matrix);

	//	get the posisiotn
	pos.x	=	matrix[12];
	pos.y	=	matrix[13];
	pos.z	=	matrix[14];

	//	get its magnitude
	float mag=pos.mag();

	//	get the looking direction by multipying location by the z vector
	camera.x = mag * matrix[2];
	camera.y = mag * matrix[6];
	camera.z = mag * matrix[10];


	//	adjust pointers
	vert		=	m_Verts.begin()	;
	color		=	m_Colors.begin()	;
	normal		=	m_Norms.begin()	;
	texCoord	=	(float*)m_TexCoords.begin();

	mInc = 1.0f/(m-1.0f);
	nInc = 1.0f/(n-1.0f);


	//	loop on all vertices
	for (i=0; i<n; i++)
	{
		for (j=0; j<m; j++)
		{
			float tmpf;
			float tmpf2;
			point3d eyeVec;
			point3d norm;

			point3d tmpv;
			float dist[5];
			int tmpi;


			//	copute refraction vectors
			//	Get Eye vector to this vertex
			eyeVec.x = camera.x - vert->x;
			eyeVec.y = camera.y - vert->y;
			eyeVec.z = camera.z - vert->z;
			
			tmpf = 1.0f / (float)sqrt(eyeVec.x*eyeVec.x + eyeVec.y*eyeVec.y + eyeVec.z*eyeVec.z);
			eyeVec.x *= tmpf;
			eyeVec.y *= tmpf;
			eyeVec.z *= tmpf;

			//	compute normals
			tmpf = normal->x*eyeVec.x + normal->y*eyeVec.y + normal->z*eyeVec.z;
			tmpf2 = 1.0f - ((0.75f*0.75f) * (1.0f-(tmpf*tmpf)));
			tmpf2 = (float)sqrt(tmpf2);

			tmpf2 = (0.75f*tmpf) - tmpf2;
			norm.x = (tmpf2*normal->x) - (0.75f*eyeVec.x);
			norm.y = (tmpf2*normal->y) - (0.75f*eyeVec.y);
			norm.z = (tmpf2*normal->z) - (0.75f*eyeVec.z);
			tmpf2 = 1.0f / (float)sqrt(norm.x*norm.x + norm.y*norm.y + norm.z*norm.z);
			
			norm.x *= tmpf2;
			norm.y *= tmpf2;
			norm.z *= tmpf2;

			//	calculate fresnel terms, they change the water colors according to the view point
			if (m_Fresnel)
				color[0] = (unsigned char)((1.0f - (tmpf*0.8f + 0.1f)) * 255.0f);
			else
				color[0] = 200;

				color[1] = color[0];
				color[2] = color[0];
				color[3] = color[0];

			//	now we start calcuating the texture coordiantes for the
			//	refractiom map.
			
			//Point on surface
			tmpv.x = ((j*mInc)*0.998f+0.001f) * WATER_ASPECT;
			tmpv.y = (i*nInc)*0.998f+0.001f;
			tmpv.z = 0.0f;

			/***************************************/
			/* Intersect with left plane (-1 0 0 0) */
			/***************************************/
			if (norm.x == 0.0f) //Parallel to plane
				dist[0] = 999999999.9f;
			else
				dist[0] = (tmpv.x) / (-norm.x);
			if (dist[0] < 0.0f) //If behind ray
				dist[0] = 999999999.9f;

			/****************************************/
			/* Intersect with right plane (-1 0 0 -1) */
			/****************************************/
			if (norm.x == 0.0f) //Parallel to plane
				dist[1] = 999999999.9f;
			else
				dist[1] = (-WATER_ASPECT + tmpv.x) / (-norm.x);
			if (dist[1] < 0.0f) //If behind ray
				dist[1] = 999999999.9f;

			/*****************************************/
			/* Intersect with bottom plane (0 -1 0 0) */
			/*****************************************/
			if (norm.y == 0.0f) //Parallel to plane
				dist[2] = 999999999.9f;
			else
				dist[2] = (tmpv.y) / (-norm.y);
			if (dist[2] < 0.0f) //If behind ray
				dist[2] = 999999999.9f;

			/***************************************/
			/* Intersect with top plane (0 -1 0 -1)*/
			/***************************************/
			if (norm.y == 0.0f) //Parallel to plane
				dist[3] = 999999999.9f;
			else
				dist[3] = (-1.0f + tmpv.y) / (-norm.y);
			if (dist[3] < 0.0f) //If behind ray
				dist[3] = 999999999.9f;

			/****************************************/
			/* Intersect with floor plane (0 0 -1 1) */
			/****************************************/
			dist[4] = (m_Depth + tmpv.z) / (-norm.z);
			if (dist[4] < 0.0f) //If behind ray
				dist[4] = 999999999.9f;

			/*********************/
			/* Find closest wall */
			/*********************/
			tmpi = 0;
			if (dist[1] < dist[tmpi])
				tmpi = 1;
			if (dist[2] < dist[tmpi])
				tmpi = 2;
			if (dist[3] < dist[tmpi])
				tmpi = 3;
			if (dist[4] < dist[tmpi])
				tmpi = 4;

			/*********/
			/* Floor */
			/*********/
			if (tmpi == 4)
			{
				tmpv.x += norm.x*dist[tmpi];
				tmpv.y += norm.y*dist[tmpi];
				tmpv.z += norm.z*dist[tmpi];

				texCoord[0] = tmpv.x/WATER_ASPECT*0.5f + 0.25f;
				texCoord[1] = tmpv.y*0.5f + 0.25f;
			}
			/*************/
			/* Left Wall */
			/*************/
			else if (tmpi == 0)
			{
				tmpv.x += norm.x*dist[tmpi];
				tmpv.y += norm.y*dist[tmpi];
				tmpv.z = norm.z*dist[tmpi];

				//Use y and z to figure out texture coords
				texCoord[0] = -0.25f*tmpv.z/m_Depth;
				texCoord[1] = ((texCoord[0])) + (tmpv.y*((1.0f-2.0f*texCoord[0])));
			}
			/**************/
			/* Right Wall */
			/**************/
			else if (tmpi == 1)
			{
				tmpv.x += norm.x*dist[tmpi];
				tmpv.y += norm.y*dist[tmpi];
				tmpv.z = norm.z*dist[tmpi];

				//Use y and z to figure out texture coords
				texCoord[0] = -0.25f*tmpv.z/m_Depth;
				texCoord[1] = ((texCoord[0])) + (tmpv.y*((1.0f-2.0f*texCoord[0])));
				texCoord[0] = 1.0f - texCoord[0];
			}
			/***************/
			/* Bottom Wall */
			/***************/
			else if (tmpi == 2)
			{
				tmpv.x += norm.x*dist[tmpi];
				tmpv.y += norm.y*dist[tmpi];
				tmpv.z = norm.z*dist[tmpi];

				//Use x and z to figure out texture coords
				texCoord[1] = -0.25f*tmpv.z/m_Depth;
				texCoord[0] = ((texCoord[1])) + (tmpv.x/WATER_ASPECT*((1.0f-2.0f*texCoord[1])));
			}
			/************/
			/* Top Wall */
			/************/
			else if (tmpi == 3)
			{
				tmpv.x += norm.x*dist[tmpi];
				tmpv.y += norm.y*dist[tmpi];
				tmpv.z = norm.z*dist[tmpi];

				//Use x and z to figure out texture coords
				texCoord[1] = -0.25f*tmpv.z/m_Depth;
				texCoord[0] = ((texCoord[1])) + (tmpv.x/WATER_ASPECT*((1.0f-2.0f*texCoord[1])));
				texCoord[1] = 1.0f - texCoord[1];
			}
			vert++;
			normal++;
			color += 4;
			texCoord += 2;
		}
	}
}


//	draw the water in a single pass using a multitexture
void sRealWater::DrawMultiTex(void)
{
		//Enable first multitexture stage
		glActiveTexture(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);
		glPushAttrib(GL_TEXTURE_BIT);
	        
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_PRIMARY_COLOR_EXT);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND1_RGB_EXT, GL_SRC_ALPHA);
		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_MODULATE);
		glTexEnvf(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 1.0f);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_EXT, GL_PRIMARY_COLOR_EXT);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_EXT, GL_SRC_ALPHA);
		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_EXT, GL_REPLACE);
		glTexEnvf(GL_TEXTURE_ENV, GL_ALPHA_SCALE, 1.0f);

		SetTextureMatrix(1.0f);
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		glEnable(GL_TEXTURE_GEN_R);

		glBindTexture(GL_TEXTURE_2D,m_pReflectionTexture->m_TextureName);


		//Enable second multitexture stage
		glActiveTexture(GL_TEXTURE1_ARB);
		glEnable(GL_TEXTURE_2D);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_PREVIOUS_EXT);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND1_RGB_EXT, GL_SRC_COLOR);
		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_ADD);
		glTexEnvf(GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, 1.0f);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_EXT, GL_PREVIOUS_EXT);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_EXT, GL_SRC_ALPHA);
		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_EXT, GL_REPLACE);
		glTexEnvf(GL_TEXTURE_ENV, GL_ALPHA_SCALE, 1.0f);

		glBindTexture(GL_TEXTURE_2D,m_pRefractionTexture->m_TextureName);
		

		//Set up arrays
		glVertexPointer(3, GL_FLOAT, sizeof(point3d), m_Verts.begin());
		glNormalPointer(GL_FLOAT, sizeof(point3d), m_Norms.begin());


		glClientActiveTexture(GL_TEXTURE1_ARB);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, sizeof(float)*2, m_TexCoords.begin());


		glEnableClientState(GL_COLOR_ARRAY);
		glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(unsigned char)*4, m_Colors.begin());

	  	  	  
		//Lock Arrays
		if (Graphics()->m_bCanLockArrays)         glLockArrays(0, m_NumVertices);
	  
		//Draw entire surface
		glDrawElements(GL_QUADS, m_NumIndices, GL_UNSIGNED_INT, m_Indices.begin());

		//Unlock Arrays
		if (Graphics()->m_bCanLockArrays)         glUnlockArrays();

		//Disable arrays
		glClientActiveTexture(GL_TEXTURE1_ARB);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glClientActiveTexture(GL_TEXTURE0_ARB);
		glDisableClientState(GL_COLOR_ARRAY);

		glActiveTexture(GL_TEXTURE1_ARB);
		glDisable(GL_TEXTURE_2D);
      
		glActiveTexture(GL_TEXTURE0_ARB);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_R);
		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glMatrixMode(GL_MODELVIEW);

	
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glPopAttrib();

}

//	draw using multipass
void sRealWater::DrawMultiPass(void)
{
	//	first pass, draw the refcattion map, no blending..
	glBindTexture(GL_TEXTURE_2D,m_pRefractionTexture->m_TextureName);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	//	disable normal arrays
	glDisableClientState(GL_NORMAL_ARRAY);

	//	we need vertex arrays and texture coordinates arrays here
	glVertexPointer(3, GL_FLOAT, sizeof(point3d), m_Verts.begin());

	glTexCoordPointer(2, GL_FLOAT, sizeof(float)*2, m_TexCoords.begin());

	//	Lock the vertices (only if supported)
	if (Graphics()->m_bCanLockArrays)glLockArrays(0, m_NumVertices);

	//Draw entire surface
	glDrawElements(GL_QUADS, m_NumIndices, GL_UNSIGNED_INT, m_Indices.begin());

	//Unlock Arrays
	if (Graphics()->m_bCanLockArrays)glUnlockArrays();

	//Disable arrays
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	
	//	Second Pass, blending and using an environment map 
	glBindTexture(GL_TEXTURE_2D,m_pReflectionTexture->m_TextureName);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);

	//	i need vertex arrays..
	glVertexPointer(3, GL_FLOAT, sizeof(point3d), m_Verts.begin());

	//	and normal array
	glEnableClientState(GL_NORMAL_ARRAY);
	glNormalPointer(GL_FLOAT, sizeof(point3d),m_Norms.begin());

	//	and a color array.. colors do change
	glEnableClientState(GL_COLOR_ARRAY);
	glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(unsigned char)*4, m_Colors.begin());

	//	generate texture coordiantes automaticly for the reflection..
	SetTextureMatrix(1.0f);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);

	//	Lock the arrays if this is supported
	if (Graphics()->m_bCanLockArrays)	glLockArrays(0, m_NumVertices);

	//Draw entire surface
	glDrawElements(GL_QUADS, m_NumIndices, GL_UNSIGNED_INT, m_Indices.begin());

	//Unlock Arrays
	if (Graphics()->m_bCanLockArrays)	glUnlockArrays();

	//	Disable arrays..and restore everything back
	glDisableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);

	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
}


void sRealWater::Draw(void)
{
	//Update and draw water
	CalculateRefractionCoordinates();

	if(Graphics()->m_bSupportReflectionMap)
	{
		if(Graphics()->m_bCanMultiTexture)
		{
			glActiveTexture(GL_TEXTURE0_ARB);
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);

			glActiveTexture(GL_TEXTURE1_ARB);
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);

			glActiveTexture(GL_TEXTURE0_ARB);
		}
		else
		{
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
		}
	}

	//	if the card supports multitextures and texture envcombine...draw using multitextures
	if (Graphics()->m_bCanMultiTexture&&Graphics()->m_bSupportTexEnvCombine)
		DrawMultiTex();
	else //use a 2 pass...multipass
		DrawMultiPass();
}

/*-------------end of sWaterContainer implementation------------*/